@@ -49,6 +49,7 @@ Archive::Archive(RAROptions *InitCmd)
   NewArchive=false;
 
   SilentOpen=false;
+
 }
 
 
@@ -67,9 +68,7 @@ void Archive::CheckArc(bool EnableBroken)
     // If FailedHeaderDecryption is set, we already reported that archive
     // password is incorrect.
     if (!FailedHeaderDecryption)
-    {
-      Log(FileName,St(MBadArc),FileName);
-    }
+      uiMsg(UIERROR_BADARCHIVE,FileName);
     ErrHandler.Exit(RARX_FATAL);
   }
 }
@@ -91,9 +90,7 @@ bool Archive::WCheckOpen(const wchar *Name)
     return false;
   if (!IsArchive(false))
   {
-#ifndef SHELL_EXT
-    Log(FileName,St(MNotRAR),FileName);
-#endif
+    uiMsg(UIERROR_BADARCHIVE,FileName);
     Close();
     return false;
   }
@@ -131,25 +128,17 @@ RARFORMAT Archive::IsSignature(const byte *D,size_t Size)
 bool Archive::IsArchive(bool EnableBroken)
 {
   Encrypted=false;
-#ifdef USE_QOPEN
-  QOpen.Unload();
-#endif
-
-  // Important if we reuse Archive object and it has virtual QOpen
-  // file position not matching real. For example, for 'l -v volname'.
-  Seek(0,SEEK_SET);
+  BrokenHeader=false; // Might be left from previous volume.
   
 #ifndef SFX_MODULE
   if (IsDevice())
   {
-#ifndef SHELL_EXT
-    Log(FileName,St(MInvalidName),FileName);
-#endif
+    uiMsg(UIERROR_INVALIDNAME,FileName,FileName);
     return false;
   }
 #endif
   if (Read(MarkHead.Mark,SIZEOF_MARKHEAD3)!=SIZEOF_MARKHEAD3)
-    return(false);
+    return false;
   SFXSize=0;
   
   RARFORMAT Type;
@@ -157,7 +146,7 @@ bool Archive::IsArchive(bool EnableBroken)
   {
     Format=Type;
     if (Format==RARFMT14)
-      Seek(0,SEEK_SET);
+      Seek(Tell()-SIZEOF_MARKHEAD3,SEEK_SET);
   }
   else
   {
@@ -185,9 +174,7 @@ bool Archive::IsArchive(bool EnableBroken)
   }
   if (Format==RARFMT_FUTURE)
   {
-#if !defined(SHELL_EXT) && !defined(SFX_MODULE)
-    Log(FileName,St(MNewRarFormat));
-#endif
+    uiMsg(UIERROR_NEWRARFORMAT,FileName);
     return false;
   }
   if (Format==RARFMT50) // RAR 5.0 signature is by one byte longer.
@@ -200,42 +187,6 @@ bool Archive::IsArchive(bool EnableBroken)
   else
     MarkHead.HeadSize=SIZEOF_MARKHEAD3;
 
-  // Skip the archive encryption header if any and read the main header.
-  while (ReadHeader()!=0 && GetHeaderType()!=HEAD_MAIN)
-    SeekToNext();
-
-  // This check allows to make RS based recovery even if password is incorrect.
-  // But we should not do it for EnableBroken or we'll get 'not RAR archive'
-  // messages when extracting encrypted archives with wrong password.
-  if (FailedHeaderDecryption && !EnableBroken)
-    return false;
-
-  SeekToNext();
-  if (BrokenHeader)
-  {
-#ifndef SHELL_EXT
-    Log(FileName,St(MMainHeaderBroken));
-#endif
-    if (!EnableBroken)
-      return false;
-  }
-
-/*
-  if (MainHead.EncryptVer>VER_UNPACK)
-  {
-#ifdef RARDLL
-    Cmd->DllError=ERAR_UNKNOWN_FORMAT;
-#else
-    ErrHandler.SetErrorCode(RARX_WARNING);
-  #if !defined(SILENT) && !defined(SFX_MODULE)
-      Log(FileName,St(MUnknownMeth),FileName);
-      Log(FileName,St(MVerRequired),MainHead.EncryptVer/10,MainHead.EncryptVer%10);
-  #endif
-#endif
-    return(false);
-  }
-*/
-
 #ifdef RARDLL
   // If callback function is not set, we cannot get the password,
   // so we skip the initial header processing for encrypted header archive.
@@ -245,43 +196,71 @@ bool Archive::IsArchive(bool EnableBroken)
     SilentOpen=true;
 #endif
 
-  MainComment=MainHead.CommentInHeader;
-
-#ifdef USE_QOPEN
-  if (MainHead.Locator && MainHead.QOpenOffset>0 && Cmd->QOpenMode!=QOPEN_NONE)
+  // Skip the archive encryption header if any and read the main header.
+  while (ReadHeader()!=0)
   {
-    QOpen.Init(this,false);
-    QOpen.Load(MainHead.QOpenOffset);
+    HEADER_TYPE Type=GetHeaderType();
+    // In RAR 5.0 we need to quit after reading HEAD_CRYPT if we wish to
+    // avoid the password prompt.
+    if (Type==HEAD_MAIN || SilentOpen && Type==HEAD_CRYPT)
+      break;
+    SeekToNext();
   }
-#endif
+
+  // This check allows to make RS based recovery even if password is incorrect.
+  // But we should not do it for EnableBroken or we'll get 'not RAR archive'
+  // messages when extracting encrypted archives with wrong password.
+  if (FailedHeaderDecryption && !EnableBroken)
+    return false;
+
+  SeekToNext();
+  if (BrokenHeader) // Main archive header is corrupt.
+  {
+    uiMsg(UIERROR_MHEADERBROKEN,FileName);
+    if (!EnableBroken)
+      return false;
+  }
+
+  MainComment=MainHead.CommentInHeader;
 
   // If we process non-encrypted archive or can request a password,
   // we set 'first volume' flag based on file attributes below.
   // It is necessary for RAR 2.x archives, which did not have 'first volume'
-  // flag in main header.
+  // flag in main header. Also for all RAR formats we need to scan until
+  // first file header to set "comment" flag when reading service header.
+  // Unless we are in silent mode, we need to know about presence of comment
+  // immediately after IsArchive call.
   if (!SilentOpen || !Encrypted)
   {
     SaveFilePos SavePos(*this);
     int64 SaveCurBlockPos=CurBlockPos,SaveNextBlockPos=NextBlockPos;
+    HEADER_TYPE SaveCurHeaderType=CurHeaderType;
 
     while (ReadHeader()!=0)
     {
       HEADER_TYPE HeaderType=GetHeaderType();
       if (HeaderType==HEAD_SERVICE)
       {
-        if (SubHead.CmpName(SUBHEAD_TYPE_CMT))
-          MainComment=true;
-        FirstVolume=!SubHead.SplitBefore;
+        // If we have a split service headers, it surely indicates non-first
+        // volume. But not split service header does not guarantee the first
+        // volume, because we can have split file after non-split archive
+        // comment. So we do not quit from loop here.
+        FirstVolume=Volume && !SubHead.SplitBefore;
       }
       else
-      {
-        FirstVolume=HeaderType==HEAD_FILE && !FileHead.SplitBefore;
-        break;
-      }
+        if (HeaderType==HEAD_FILE)
+        {
+          FirstVolume=Volume && !FileHead.SplitBefore;
+          break;
+        }
+        else
+          if (HeaderType==HEAD_ENDARC) // Might happen if archive contains only a split service header.
+            break;
       SeekToNext();
     }
     CurBlockPos=SaveCurBlockPos;
     NextBlockPos=SaveNextBlockPos;
+    CurHeaderType=SaveCurHeaderType;
   }
   if (!Volume || FirstVolume)
     wcscpy(FirstVolumeName,FileName);
@@ -320,6 +299,16 @@ uint Archive::FullHeaderSize(size_t Size)
 
 
 #ifdef USE_QOPEN
+bool Archive::Open(const wchar *Name,uint Mode)
+{
+  // Important if we reuse Archive object and it has virtual QOpen
+  // file position not matching real. For example, for 'l -v volname'.
+  QOpen.Unload();
+
+  return File::Open(Name,Mode);
+}
+
+
 int Archive::Read(void *Data,size_t Size)
 {
   size_t Result;
