@@ -1,6 +1,5 @@
 #include "rar.hpp"
 
-
 CommandData::CommandData()
 {
   Init();
@@ -14,7 +13,6 @@ void CommandData::Init()
   *Command=0;
   *ArcName=0;
   FileLists=false;
-  NoMoreSwitches=false;
 
   ListMode=RCLM_AUTO;
 
@@ -45,13 +43,16 @@ static const wchar *AllocCmdParam(const wchar *CmdLine,wchar **Par)
 }
 
 
-#ifndef SFX_MODULE
+#if !defined(SFX_MODULE) && !defined(_ANDROID)
 void CommandData::ParseCommandLine(bool Preprocess,int argc, char *argv[])
 {
+  *Command=0;
+  NoMoreSwitches=false;
 #ifdef CUSTOM_CMDLINE_PARSER
   // In Windows we may prefer to implement our own command line parser
   // to avoid replacing \" by " in standard parser. Such replacing corrupts
   // destination paths like "dest path\" in extraction commands.
+  // Also our own parser is Unicode compatible.
   const wchar *CmdLine=GetCommandLine();
 
   wchar *Par;
@@ -59,15 +60,12 @@ void CommandData::ParseCommandLine(bool Preprocess,int argc, char *argv[])
   {
     if ((CmdLine=AllocCmdParam(CmdLine,&Par))==NULL)
       break;
-    bool Code=true;
     if (!FirstParam) // First parameter is the executable name.
       if (Preprocess)
-        Code=PreprocessSwitch(Par);
+        PreprocessArg(Par);
       else
         ParseArg(Par);
     free(Par);
-    if (Preprocess && !Code)
-      break;
   }
 #else
   Array<wchar> Arg;
@@ -76,10 +74,7 @@ void CommandData::ParseCommandLine(bool Preprocess,int argc, char *argv[])
     Arg.Alloc(strlen(argv[I])+1);
     CharToWide(argv[I],&Arg[0],Arg.Size());
     if (Preprocess)
-    {
-      if (!PreprocessSwitch(&Arg[0]))
-        break;
-    }
+      PreprocessArg(&Arg[0]);
     else
       ParseArg(&Arg[0]);
   }
@@ -90,11 +85,11 @@ void CommandData::ParseCommandLine(bool Preprocess,int argc, char *argv[])
 #endif
 
 
-#ifndef SFX_MODULE
+#if !defined(SFX_MODULE) && !defined(_ANDROID)
 void CommandData::ParseArg(wchar *Arg)
 {
   if (IsSwitch(*Arg) && !NoMoreSwitches)
-    if (Arg[1]=='-')
+    if (Arg[1]=='-' && Arg[2]==0)
       NoMoreSwitches=true;
     else
       ProcessSwitch(Arg+1);
@@ -182,7 +177,7 @@ void CommandData::ParseDone()
 }
 
 
-#ifndef SFX_MODULE
+#if !defined(SFX_MODULE) && !defined(_ANDROID)
 void CommandData::ParseEnvVar()
 {
   char *EnvStr=getenv("RAR");
@@ -197,45 +192,45 @@ void CommandData::ParseEnvVar()
 
 
 
-#ifndef SFX_MODULE
+#if !defined(SFX_MODULE) && !defined(_ANDROID)
 // Preprocess those parameters, which must be processed before the rest of
 // command line. Return 'false' to stop further processing.
-bool CommandData::PreprocessSwitch(const wchar *Switch)
+void CommandData::PreprocessArg(const wchar *Arg)
 {
-  if (IsSwitch(Switch[0]))
+  if (IsSwitch(Arg[0]) && !NoMoreSwitches)
   {
-    Switch++;
-    char SwitchA[1024];
-    WideToChar(Switch,SwitchA,ASIZE(SwitchA));
-    if (wcsicomp(Switch,L"-")==0) // Switch "--".
-      return false;
-    if (wcsicomp(Switch,L"cfg-")==0)
+    Arg++;
+    if (Arg[0]=='-' && Arg[1]==0) // Switch "--".
+      NoMoreSwitches=true;
+    if (wcsicomp(Arg,L"cfg-")==0)
       ConfigDisabled=true;
 #ifndef GUI
-    if (wcsnicomp(Switch,L"ilog",4)==0)
+    if (wcsnicomp(Arg,L"ilog",4)==0)
     {
       // Ensure that correct log file name is already set
       // if we need to report an error when processing the command line.
-      ProcessSwitch(Switch);
+      ProcessSwitch(Arg);
       InitLogOptions(LogName,ErrlogCharset);
     }
 #endif
-    if (wcsnicomp(Switch,L"sc",2)==0)
+    if (wcsnicomp(Arg,L"sc",2)==0)
     {
       // Process -sc before reading any file lists.
-      ProcessSwitch(Switch);
+      ProcessSwitch(Arg);
 #ifndef GUI
       if (*LogName!=0)
         InitLogOptions(LogName,ErrlogCharset);
 #endif
     }
   }
-  return true;
+  else
+    if (*Command==0)
+      wcsncpy(Command,Arg,ASIZE(Command)); // Need for rar.ini.
 }
 #endif
 
 
-#if !defined(GUI) && !defined(SFX_MODULE)
+#if !defined(GUI) && !defined(SFX_MODULE) && !defined(_ANDROID)
 void CommandData::ReadConfig()
 {
   StringList List;
@@ -248,13 +243,29 @@ void CommandData::ReadConfig()
         Str++;
       if (wcsnicomp(Str,L"switches=",9)==0)
         ProcessSwitchesString(Str+9);
+      if (*Command!=0)
+      {
+        wchar Cmd[16];
+        wcsncpyz(Cmd,Command,ASIZE(Cmd));
+        wchar C0=toupperw(Cmd[0]);
+        wchar C1=toupperw(Cmd[1]);
+        if (C0=='I' || C0=='L' || C0=='M' || C0=='S' || C0=='V')
+          Cmd[1]=0;
+        if (C0=='R' && (C1=='R' || C1=='V'))
+          Cmd[2]=0;
+        wchar SwName[16+ASIZE(Cmd)];
+        swprintf(SwName,ASIZE(SwName),L"switches_%s=",Cmd);
+        size_t Length=wcslen(SwName);
+        if (wcsnicomp(Str,SwName,Length)==0)
+          ProcessSwitchesString(Str+Length);
+      }
     }
   }
 }
 #endif
 
 
-#ifndef SFX_MODULE
+#if !defined(SFX_MODULE) && !defined(_ANDROID)
 void CommandData::ProcessSwitchesString(const wchar *Str)
 {
   wchar *Par;
@@ -268,7 +279,7 @@ void CommandData::ProcessSwitchesString(const wchar *Str)
 #endif
 
 
-#if !defined(SFX_MODULE)
+#if !defined(SFX_MODULE) && !defined(_ANDROID)
 void CommandData::ProcessSwitch(const wchar *Switch)
 {
 
@@ -366,17 +377,14 @@ void CommandData::ProcessSwitch(const wchar *Switch)
               break;
           }
           break;
-        case 'E':
-          ProcessEA=false;
-          break;
         default:
           if (Switch[1]=='+')
           {
-            InclFileAttr=GetExclAttr(Switch+2);
+            InclFileAttr|=GetExclAttr(Switch+2);
             InclAttrSet=true;
           }
           else
-            ExclFileAttr=GetExclAttr(Switch+1);
+            ExclFileAttr|=GetExclAttr(Switch+1);
           break;
       }
       break;
@@ -399,7 +407,7 @@ void CommandData::ProcessSwitch(const wchar *Switch)
           else
             if (!Password.IsSet())
             {
-              GetPassword(PASSWORD_GLOBAL,NULL,&Password);
+              uiGetPassword(UIPASSWORD_GLOBAL,NULL,&Password);
               eprintf(L"\n");
             }
           break;
@@ -534,7 +542,7 @@ void CommandData::ProcessSwitch(const wchar *Switch)
                 Names++;
               wchar Mask[NM];
               if (wcspbrk(Names,L"*?.")==NULL)
-                swprintf(Mask,ASIZE(Mask),L"*.%s",Names);
+                swprintf(Mask,ASIZE(Mask),L"*.%ls",Names);
               else
                 wcsncpyz(Mask,Names,ASIZE(Mask));
               StoreArgs.AddString(Mask);
@@ -609,6 +617,8 @@ void CommandData::ProcessSwitch(const wchar *Switch)
 #ifdef SAVE_LINKS
         case 'L':
           SaveSymLinks=true;
+          if (toupperw(Switch[2])=='A')
+            AbsoluteLinks=true;
           break;
 #endif
         case 'R':
@@ -630,7 +640,7 @@ void CommandData::ProcessSwitch(const wchar *Switch)
     case 'P':
       if (Switch[1]==0)
       {
-        GetPassword(PASSWORD_GLOBAL,NULL,&Password);
+        uiGetPassword(UIPASSWORD_GLOBAL,NULL,&Password);
         eprintf(L"\n");
       }
       else
@@ -757,6 +767,9 @@ void CommandData::ProcessSwitch(const wchar *Switch)
                       case 'L':
                         FilelistCharset=rch;
                         break;
+                      case 'R':
+                        RedirectCharset=rch;
+                        break;
                       default:
                         BadSwitch(Switch);
                         AlreadyBad=true;
@@ -885,7 +898,7 @@ void CommandData::ProcessSwitch(const wchar *Switch)
 #endif
 
 
-#ifndef SFX_MODULE
+#if !defined(SFX_MODULE) && !defined(_ANDROID)
 void CommandData::BadSwitch(const wchar *Switch)
 {
   mprintf(St(MUnknownOption),Switch);
@@ -931,7 +944,7 @@ inline bool CmpMSGID(MSGID i1,MSGID i2)
   // If MSGID is const char*, we cannot compare pointers only.
   // Pointers to different instances of same string can differ,
   // so we need to compare complete strings.
-  return strcmp(i1,i2)==0;
+  return wcscmp(i1,i2)==0;
 #endif
 }
 
@@ -951,10 +964,11 @@ void CommandData::OutHelp(RAR_EXIT ExitCode)
     MCHelpSwCm,MCHelpSwCFGm,MCHelpSwCL,MCHelpSwCU,
     MCHelpSwDH,MCHelpSwEP,MCHelpSwEP3,MCHelpSwF,MCHelpSwIDP,MCHelpSwIERR,
     MCHelpSwINUL,MCHelpSwIOFF,MCHelpSwKB,MCHelpSwN,MCHelpSwNa,MCHelpSwNal,
-    MCHelpSwO,MCHelpSwOC,MCHelpSwOR,MCHelpSwOW,MCHelpSwP,
-    MCHelpSwPm,MCHelpSwR,MCHelpSwRI,MCHelpSwSL,MCHelpSwSM,MCHelpSwTA,
-    MCHelpSwTB,MCHelpSwTN,MCHelpSwTO,MCHelpSwTS,MCHelpSwU,MCHelpSwVUnr,
-    MCHelpSwVER,MCHelpSwVP,MCHelpSwX,MCHelpSwXa,MCHelpSwXal,MCHelpSwY
+    MCHelpSwO,MCHelpSwOC,MCHelpSwOL,MCHelpSwOR,MCHelpSwOW,MCHelpSwP,
+    MCHelpSwPm,MCHelpSwR,MCHelpSwRI,MCHelpSwSC,MCHelpSwSL,MCHelpSwSM,
+    MCHelpSwTA,MCHelpSwTB,MCHelpSwTN,MCHelpSwTO,MCHelpSwTS,MCHelpSwU,
+    MCHelpSwVUnr,MCHelpSwVER,MCHelpSwVP,MCHelpSwX,MCHelpSwXa,MCHelpSwXal,
+    MCHelpSwY
 #else
 #endif
   };
@@ -995,15 +1009,6 @@ void CommandData::OutHelp(RAR_EXIT ExitCode)
     if (CmpMSGID(Help[I],MCHelpSwMT))
       continue;
 #endif
-    if (CmpMSGID(Help[I],MCHelpSwEE))
-    {
-#if defined(_EMX) && !defined(_DJGPP)
-      if (_osmode != OS2_MODE)
-        continue;
-#else
-      continue;
-#endif
-    }
 #endif
     mprintf(St(Help[I]));
   }
@@ -1018,17 +1023,17 @@ void CommandData::OutHelp(RAR_EXIT ExitCode)
 // the include list created with -n switch.
 bool CommandData::ExclCheck(const wchar *CheckName,bool Dir,bool CheckFullPath,bool CheckInclList)
 {
-  if (ExclCheckArgs(&ExclArgs,Dir,CheckName,CheckFullPath,MATCH_WILDSUBPATH))
+  if (CheckArgs(&ExclArgs,Dir,CheckName,CheckFullPath,MATCH_WILDSUBPATH))
     return true;
   if (!CheckInclList || InclArgs.ItemsCount()==0)
     return false;
-  if (ExclCheckArgs(&InclArgs,Dir,CheckName,false,MATCH_WILDSUBPATH))
+  if (CheckArgs(&InclArgs,Dir,CheckName,CheckFullPath,MATCH_WILDSUBPATH))
     return false;
   return true;
 }
 
 
-bool CommandData::ExclCheckArgs(StringList *Args,bool Dir,const wchar *CheckName,bool CheckFullPath,int MatchMode)
+bool CommandData::CheckArgs(StringList *Args,bool Dir,const wchar *CheckName,bool CheckFullPath,int MatchMode)
 {
   wchar *Name=ConvertPath(CheckName,NULL);
   wchar FullName[NM];
@@ -1063,9 +1068,9 @@ bool CommandData::ExclCheckArgs(StringList *Args,bool Dir,const wchar *CheckName
     {
       // If we process a file inside of directory excluded by "dirmask\".
       // we want to exclude such file too. So we convert "dirmask\" to
-      // "dirmask\*". It is important for operations other than archiving.
-      // When archiving, directory matched by "dirmask\" is excluded
-      // from further scanning.
+      // "dirmask\*". It is important for operations other than archiving
+      // with -x. When archiving with -x, directory matched by "dirmask\"
+      // is excluded from further scanning.
 
       if (DirMask)
         wcscat(CurMask,L"*");
@@ -1089,7 +1094,12 @@ bool CommandData::ExclCheckArgs(StringList *Args,bool Dir,const wchar *CheckName
 #endif
     {
       wchar NewName[NM+2],*CurName=Name;
-      if (CurMask[0]=='*' && IsPathDiv(CurMask[1]))
+
+      // Important to convert before "*\" check below, so masks like
+      // d:*\something are processed properly.
+      wchar *CmpMask=ConvertPath(CurMask,NULL);
+
+      if (CmpMask[0]=='*' && IsPathDiv(CmpMask[1]))
       {
         // We want "*\name" to match 'name' not only in subdirectories,
         // but also in the current directory. We convert the name
@@ -1101,7 +1111,7 @@ bool CommandData::ExclCheckArgs(StringList *Args,bool Dir,const wchar *CheckName
         CurName=NewName;
       }
 
-      if (CmpName(ConvertPath(CurMask,NULL),CurName,MatchMode))
+      if (CmpName(CmpMask,CurName,MatchMode))
         return true;
     }
   }
@@ -1158,8 +1168,11 @@ bool CommandData::SizeCheck(int64 Size)
 
 
 
-int CommandData::IsProcessFile(FileHeader &FileHead,bool *ExactMatch,int MatchType)
+int CommandData::IsProcessFile(FileHeader &FileHead,bool *ExactMatch,int MatchType,
+                               wchar *MatchedArg,uint MatchedArgSize)
 {
+  if (MatchedArg!=NULL && MatchedArgSize>0)
+    *MatchedArg=0;
   if (wcslen(FileHead.FileName)>=NM)
     return 0;
   bool Dir=FileHead.Dir;
@@ -1180,6 +1193,8 @@ int CommandData::IsProcessFile(FileHeader &FileHead,bool *ExactMatch,int MatchTy
     {
       if (ExactMatch!=NULL)
         *ExactMatch=wcsicompc(ArgName,FileHead.FileName)==0;
+      if (MatchedArg!=NULL)
+        wcsncpyz(MatchedArg,ArgName,MatchedArgSize);
       return StringCount;
     }
   return 0;
@@ -1228,7 +1243,7 @@ void CommandData::ProcessCommand()
     case 'I':
       {
         CmdExtract Extract(this);
-        Extract.DoExtract(this);
+        Extract.DoExtract();
       }
       break;
 #ifndef SILENT
@@ -1334,37 +1349,23 @@ void CommandData::ReportWrongSwitches(RARFORMAT Format)
   if (Format==RARFMT15)
   {
     if (HashType!=HASH_CRC32)
-    {
-      mprintf(St(MIncompatSwitch),L"-ht",4);
-    }
+      uiMsg(UIERROR_INCOMPATSWITCH,L"-ht",4);
 #ifdef _WIN_ALL
     if (SaveSymLinks)
-    {
-      mprintf(St(MIncompatSwitch),L"-ol",4);
-    }
+      uiMsg(UIERROR_INCOMPATSWITCH,L"-ol",4);
 #endif
     if (SaveHardLinks)
-    {
-      mprintf(St(MIncompatSwitch),L"-oh",4);
-    }
+      uiMsg(UIERROR_INCOMPATSWITCH,L"-oh",4);
 
 #ifdef _WIN_ALL
+    // Do not report a wrong dictionary size here, because we are not sure
+    // yet about archive format. We can switch to RAR5 mode later
+    // if we update RAR5 archive.
 
 
 #endif
     if (QOpenMode!=QOPEN_AUTO)
-    {
-      mprintf(St(MIncompatSwitch),L"-qo",4);
-    }
-/*
-    // We use 64 MB for both formats and reduce it for RAR 4.x later.
-    if (WinSize>0x400000)
-    {
-      wchar SwMD[10];
-      swprintf(SwMD,ASIZE(SwMD),L"-md%dm",WinSize/0x100000);
-      mprintf(St(MIncompatSwitch),SwMD,4);
-    }
-*/
+      uiMsg(UIERROR_INCOMPATSWITCH,L"-qo",4);
   }
   if (Format==RARFMT50)
   {
