@@ -2,7 +2,8 @@
 #include "log.cpp"
 
 static MESSAGE_TYPE MsgStream=MSG_STDOUT;
-static bool Sound=false;
+static RAR_CHARSET RedirectCharset=RCH_DEFAULT;
+
 const int MaxMsgSize=2*NM+2048;
 
 #ifdef _WIN_ALL
@@ -47,10 +48,10 @@ void InitConsole()
 }
 
 
-void InitConsoleOptions(MESSAGE_TYPE MsgStream,bool Sound)
+void InitConsoleOptions(MESSAGE_TYPE MsgStream,RAR_CHARSET RedirectCharset)
 {
   ::MsgStream=MsgStream;
-  ::Sound=Sound;
+  ::RedirectCharset=RedirectCharset;
 }
 
 
@@ -65,17 +66,23 @@ static void cvt_wprintf(FILE *dest,const wchar *fmt,va_list arglist)
   safebuf wchar Msg[MaxMsgSize];
   if (dest==stdout && StdoutRedirected || dest==stderr && StderrRedirected)
   {
-    // Avoid Unicode for redirect in Windows, it does not work with pipes.
-    vswprintf(Msg,ASIZE(Msg),fmtw,arglist);
-    safebuf char MsgA[MaxMsgSize];
-    WideToChar(Msg,MsgA,ASIZE(MsgA));
-    CharToOemA(MsgA,MsgA); // Console tools like 'more' expect OEM encoding.
-
-    // We already converted \n to \r\n above, so we use WriteFile instead
-    // of C library to avoid unnecessary additional conversion.
     HANDLE hOut=GetStdHandle(dest==stdout ? STD_OUTPUT_HANDLE:STD_ERROR_HANDLE);
+    vswprintf(Msg,ASIZE(Msg),fmtw,arglist);
     DWORD Written;
-    WriteFile(hOut,MsgA,(DWORD)strlen(MsgA),&Written,NULL);
+    if (RedirectCharset==RCH_UNICODE)
+      WriteFile(hOut,Msg,(DWORD)wcslen(Msg)*sizeof(*Msg),&Written,NULL);
+    else
+    {
+      // Avoid Unicode for redirect in Windows, it does not work with pipes.
+      safebuf char MsgA[MaxMsgSize];
+      WideToChar(Msg,MsgA,ASIZE(MsgA));
+      if (RedirectCharset!=RCH_ANSI)
+        CharToOemA(MsgA,MsgA); // Console tools like 'more' expect OEM encoding.
+
+      // We already converted \n to \r\n above, so we use WriteFile instead
+      // of C library to avoid unnecessary additional conversion.
+      WriteFile(hOut,MsgA,(DWORD)strlen(MsgA),&Written,NULL);
+    }
     return;
   }
   // MSVC2008 vfwprintf writes every character to console separately
@@ -124,25 +131,6 @@ void eprintf(const wchar *fmt,...)
 #endif
 
 
-#ifndef SILENT
-void Alarm()
-{
-  if (Sound)
-  {
-    static clock_t LastTime=clock();
-    if ((clock()-LastTime)/CLOCKS_PER_SEC>5)
-    {
-#ifdef _WIN_ALL
-      MessageBeep(-1);
-#else
-      putwchar('\007');
-#endif
-    }
-  }
-}
-#endif
-
-
 #ifndef SILENT
 static void GetPasswordText(wchar *Str,uint MaxLength)
 {
@@ -181,22 +169,22 @@ static void GetPasswordText(wchar *Str,uint MaxLength)
 
 
 #ifndef SILENT
-bool GetConsolePassword(PASSWORD_TYPE Type,const wchar *FileName,SecPassword *Password)
+bool GetConsolePassword(UIPASSWORD_TYPE Type,const wchar *FileName,SecPassword *Password)
 {
-  Alarm();
+  uiAlarm(UIALARM_QUESTION);
   
   while (true)
   {
-    if (Type==PASSWORD_GLOBAL)
+    if (Type==UIPASSWORD_GLOBAL)
       eprintf(L"\n%s: ",St(MAskPsw));
     else
       eprintf(St(MAskPswFor),FileName);
 
     wchar PlainPsw[MAXPASSWORD];
     GetPasswordText(PlainPsw,ASIZE(PlainPsw));
-    if (*PlainPsw==0 && Type==PASSWORD_GLOBAL)
+    if (*PlainPsw==0 && Type==UIPASSWORD_GLOBAL)
       return false;
-    if (Type==PASSWORD_GLOBAL)
+    if (Type==UIPASSWORD_GLOBAL)
     {
       eprintf(St(MReAskPsw));
       wchar CmpStr[MAXPASSWORD];
@@ -264,9 +252,13 @@ bool getwstr(wchar *str,size_t n)
 
 
 #ifndef SILENT
+// We allow this function to return 0 in case of invalid input,
+// because it might be convenient to press Enter to some not dangerous
+// prompts like "insert disk with next volume". We should call this function
+// again in case of 0 in dangerous prompt such as overwriting file.
 int Ask(const wchar *AskStr)
 {
-  Alarm();
+  uiAlarm(UIALARM_QUESTION);
 
   const int MaxItems=10;
   wchar Item[MaxItems][40];
