@@ -2,7 +2,7 @@
 #define _RAR_FILECREATE_
 
 bool FileCreate(RAROptions *Cmd,File *NewFile,wchar *Name,size_t MaxNameSize,
-                OVERWRITE_MODE Mode,bool *UserReject,int64 FileSize=INT64NDF,
+                bool *UserReject,int64 FileSize=INT64NDF,
                 RarTime *FileTime=NULL,bool WriteOnly=false);
 
 bool GetAutoRenamedName(wchar *Name,size_t MaxNameSize);
