@@ -1,30 +1,28 @@
 bool ExtractHardlink(wchar *NameNew,wchar *NameExisting,size_t NameExistingSize)
 {
+  SlashToNative(NameExisting,NameExisting,NameExistingSize); // Not needed for RAR 5.1+ archives.
+
   if (!FileExist(NameExisting))
     return false;
   CreatePath(NameNew,true);
 
 #ifdef _WIN_ALL
-  UnixSlashToDos(NameExisting,NameExisting,NameExistingSize);
-
   bool Success=CreateHardLink(NameNew,NameExisting,NULL)!=0;
   if (!Success)
   {
-    Log(NULL,St(MErrCreateLnkH),NameNew);
+    uiMsg(UIERROR_HLINKCREATE,NameNew);
     ErrHandler.SysErrMsg();
     ErrHandler.SetErrorCode(RARX_CREATE);
   }
   return Success;
 #elif defined(_UNIX)
-  DosSlashToUnix(NameExisting,NameExisting,NameExistingSize);
-
   char NameExistingA[NM],NameNewA[NM];
   WideToChar(NameExisting,NameExistingA,ASIZE(NameExistingA));
   WideToChar(NameNew,NameNewA,ASIZE(NameNewA));
   bool Success=link(NameExistingA,NameNewA)==0;
   if (!Success)
   {
-    Log(NULL,St(MErrCreateLnkH),NameNew);
+    uiMsg(UIERROR_HLINKCREATE,NameNew);
     ErrHandler.SysErrMsg();
     ErrHandler.SetErrorCode(RARX_CREATE);
   }
