@@ -29,7 +29,7 @@ wchar* ConvertPath(const wchar *SrcPath,wchar *DestPath)
   while (*DestPtr!=0)
   {
     const wchar *s=DestPtr;
-    if (s[0] && IsDriveDiv(s[1]))
+    if (s[0]!=0 && IsDriveDiv(s[1]))
       s+=2;
     if (s[0]=='\\' && s[1]=='\\')
     {
@@ -71,44 +71,39 @@ void SetName(wchar *FullName,const wchar *Name,size_t MaxSize)
 }
 
 
-void SetExt(wchar *Name,const wchar *NewExt)
+void SetExt(wchar *Name,const wchar *NewExt,size_t MaxSize)
 {
   if (Name==NULL || *Name==0)
     return;
   wchar *Dot=GetExt(Name);
-  if (NewExt==NULL)
+  if (Dot!=NULL)
+    *Dot=0;
+  if (NewExt!=NULL)
   {
-    if (Dot!=NULL)
-      *Dot=0;
+    wcsncatz(Name,L".",MaxSize);
+    wcsncatz(Name,NewExt,MaxSize);
   }
-  else
-    if (Dot==NULL)
-    {
-      wcscat(Name,L".");
-      wcscat(Name,NewExt);
-    }
-    else
-      wcscpy(Dot+1,NewExt);
 }
 
 
 #ifndef SFX_MODULE
-void SetSFXExt(wchar *SFXName)
+void SetSFXExt(wchar *SFXName,size_t MaxSize)
 {
   if (SFXName==NULL || *SFXName==0)
     return;
 
 #ifdef _UNIX
-  SetExt(SFXName,L"sfx");
+  SetExt(SFXName,L"sfx",MaxSize);
 #endif
 
 #if defined(_WIN_ALL) || defined(_EMX)
-  SetExt(SFXName,L"exe");
+  SetExt(SFXName,L"exe",MaxSize);
 #endif
 }
 #endif
 
 
+// 'Ext' is an extension with the leading dot, like L".rar".
 wchar *GetExt(const wchar *Name)
 {
   return Name==NULL ? NULL:wcsrchr(PointToName(Name),'.');
@@ -149,9 +144,16 @@ bool IsDriveDiv(int Ch)
 }
 
 
+bool IsDriveLetter(const wchar *Path)
+{
+  wchar Letter=etoupperw(Path[0]);
+  return Letter>='A' && Letter<='Z' && IsDriveDiv(Path[1]);
+}
+
+
 int GetPathDisk(const wchar *Path)
 {
-  if (IsDiskLetter(Path))
+  if (IsDriveLetter(Path))
     return etoupperw(*Path)-'A';
   else
     return -1;
@@ -162,7 +164,7 @@ void AddEndSlash(wchar *Path,size_t MaxLength)
 {
   size_t Length=wcslen(Path);
   if (Length>0 && Path[Length-1]!=CPATHDIVIDER && Length+1<MaxLength)
-    wcscat(Path,PATHDIVIDERW);
+    wcscat(Path,SPATHDIVIDER);
 }
 
 
@@ -201,7 +203,7 @@ void RemoveNameFromPath(wchar *Path)
 
 
 #if defined(_WIN_ALL) && !defined(SFX_MODULE)
-static void GetAppDataPath(wchar *Path,size_t MaxSize,bool Create)
+bool GetAppDataPath(wchar *Path,size_t MaxSize,bool Create)
 {
   LPMALLOC g_pMalloc;
   SHGetMalloc(&g_pMalloc);
@@ -217,12 +219,8 @@ static void GetAppDataPath(wchar *Path,size_t MaxSize,bool Create)
     if (!Success && Create)
       Success=MakeDir(Path,false,0)==MKDIR_SUCCESS;
   }
-  if (!Success)
-  {
-    GetModuleFileName(NULL,Path,(DWORD)MaxSize);
-    RemoveNameFromPath(Path);
-  }
   g_pMalloc->Free(ppidl);
+  return Success;
 }
 #endif
 
@@ -242,7 +240,11 @@ void GetRarDataPath(wchar *Path,size_t MaxSize,bool Create)
   }
 
   if (*Path==0 || !FileExist(Path))
-    GetAppDataPath(Path,MaxSize,Create);
+    if (!GetAppDataPath(Path,MaxSize,Create))
+    {
+      GetModuleFileName(NULL,Path,(DWORD)MaxSize);
+      RemoveNameFromPath(Path);
+    }
 }
 #endif
 
@@ -424,7 +426,8 @@ void MakeNameUsable(char *Name,bool Extended)
 #ifndef _UNIX
     if (s-Name>1 && *s==':')
       *s='_';
-    if ((*s==' ' || *s=='.') && IsPathDiv(s[1]))
+    // Remove ' ' and '.' before path separator, but allow .\ and ..\.
+    if ((*s==' ' || *s=='.' && s>Name && !IsPathDiv(s[-1]) && s[-1]!='.') && IsPathDiv(s[1]))
       *s='_';
 #endif
   }
@@ -440,98 +443,50 @@ void MakeNameUsable(wchar *Name,bool Extended)
 #ifndef _UNIX
     if (s-Name>1 && *s==':')
       *s='_';
-    if ((*s==' ' || *s=='.') && IsPathDiv(s[1]))
+#if 0  // We already can create such files.
+    // Remove ' ' and '.' before path separator, but allow .\ and ..\.
+    if (IsPathDiv(s[1]) && (*s==' ' || *s=='.' && s>Name &&
+        !IsPathDiv(s[-1]) && (s[-1]!='.' || s>Name+1 && !IsPathDiv(s[-2]))))
       *s='_';
+#endif
 #endif
   }
 }
 
 
-char* UnixSlashToDos(char *SrcName,char *DestName,size_t MaxLength)
+void UnixSlashToDos(const char *SrcName,char *DestName,size_t MaxLength)
 {
-  if (DestName!=NULL && DestName!=SrcName)
-    if (strlen(SrcName)>=MaxLength)
-    {
-      *DestName=0;
-      return DestName;
-    }
-    else
-      strcpy(DestName,SrcName);
-  for (char *s=SrcName;*s!=0;s=charnext(s))
-  {
-    if (*s=='/')
-      if (DestName==NULL)
-        *s='\\';
-      else
-        DestName[s-SrcName]='\\';
-  }
-  return DestName==NULL ? SrcName:DestName;
+  size_t Copied=0;
+  for (;Copied<MaxLength-1 && SrcName[Copied]!=0;Copied++)
+    DestName[Copied]=SrcName[Copied]=='/' ? '\\':SrcName[Copied];
+  DestName[Copied]=0;
 }
 
 
-char* DosSlashToUnix(char *SrcName,char *DestName,size_t MaxLength)
+void DosSlashToUnix(const char *SrcName,char *DestName,size_t MaxLength)
 {
-  if (DestName!=NULL && DestName!=SrcName)
-    if (strlen(SrcName)>=MaxLength)
-    {
-      *DestName=0;
-      return DestName;
-    }
-    else
-      strcpy(DestName,SrcName);
-  for (char *s=SrcName;*s!=0;s=charnext(s))
-  {
-    if (*s=='\\')
-      if (DestName==NULL)
-        *s='/';
-      else
-        DestName[s-SrcName]='/';
-  }
-  return DestName==NULL ? SrcName:DestName;
+  size_t Copied=0;
+  for (;Copied<MaxLength-1 && SrcName[Copied]!=0;Copied++)
+    DestName[Copied]=SrcName[Copied]=='\\' ? '/':SrcName[Copied];
+  DestName[Copied]=0;
 }
 
 
-wchar* UnixSlashToDos(wchar *SrcName,wchar *DestName,size_t MaxLength)
+void UnixSlashToDos(const wchar *SrcName,wchar *DestName,size_t MaxLength)
 {
-  if (DestName!=NULL && DestName!=SrcName)
-    if (wcslen(SrcName)>=MaxLength)
-    {
-      *DestName=0;
-      return DestName;
-    }
-    else
-      wcscpy(DestName,SrcName);
-  for (wchar *s=SrcName;*s!=0;s++)
-  {
-    if (*s=='/')
-      if (DestName==NULL)
-        *s='\\';
-      else
-        DestName[s-SrcName]='\\';
-  }
-  return DestName==NULL ? SrcName:DestName;
+  size_t Copied=0;
+  for (;Copied<MaxLength-1 && SrcName[Copied]!=0;Copied++)
+    DestName[Copied]=SrcName[Copied]=='/' ? '\\':SrcName[Copied];
+  DestName[Copied]=0;
 }
 
 
-wchar* DosSlashToUnix(wchar *SrcName,wchar *DestName,size_t MaxLength)
+void DosSlashToUnix(const wchar *SrcName,wchar *DestName,size_t MaxLength)
 {
-  if (DestName!=NULL && DestName!=SrcName)
-    if (wcslen(SrcName)>=MaxLength)
-    {
-      *DestName=0;
-      return DestName;
-    }
-    else
-      wcscpy(DestName,SrcName);
-  for (wchar *s=SrcName;*s!=0;s++)
-  {
-    if (*s=='\\')
-      if (DestName==NULL)
-        *s='/';
-      else
-        DestName[s-SrcName]='/';
-  }
-  return DestName==NULL ? SrcName:DestName;
+  size_t Copied=0;
+  for (;Copied<MaxLength-1 && SrcName[Copied]!=0;Copied++)
+    DestName[Copied]=SrcName[Copied]=='\\' ? '/':SrcName[Copied];
+  DestName[Copied]=0;
 }
 
 
@@ -586,24 +541,23 @@ bool IsFullPath(const wchar *Path)
     return true;
 */
 #if defined(_WIN_ALL) || defined(_EMX)
-  return Path[0]=='\\' && Path[1]=='\\' || IsDiskLetter(Path) && IsPathDiv(Path[2]);
+  return Path[0]=='\\' && Path[1]=='\\' || IsDriveLetter(Path) && IsPathDiv(Path[2]);
 #else
   return IsPathDiv(Path[0]);
 #endif
 }
 
 
-bool IsDiskLetter(const wchar *Path)
+bool IsFullRootPath(const wchar *Path)
 {
-  wchar Letter=etoupperw(Path[0]);
-  return Letter>='A' && Letter<='Z' && IsDriveDiv(Path[1]);
+  return IsFullPath(Path) || IsPathDiv(Path[0]);
 }
 
 
 void GetPathRoot(const wchar *Path,wchar *Root,size_t MaxSize)
 {
   *Root=0;
-  if (IsDiskLetter(Path))
+  if (IsDriveLetter(Path))
     swprintf(Root,MaxSize,L"%c:\\",*Path);
   else
     if (Path[0]=='\\' && Path[1]=='\\')
@@ -642,10 +596,10 @@ int ParseVersionFileName(wchar *Name,bool Truncate)
 
 #if !defined(SFX_MODULE) && !defined(SETUP)
 // Get the name of first volume. Return the leftmost digit of volume number.
-wchar* VolNameToFirstName(const wchar *VolName,wchar *FirstName,bool NewNumbering)
+wchar* VolNameToFirstName(const wchar *VolName,wchar *FirstName,size_t MaxSize,bool NewNumbering)
 {
   if (FirstName!=VolName)
-    wcscpy(FirstName,VolName);
+    wcsncpyz(FirstName,VolName,MaxSize);
   wchar *VolNumStart=FirstName;
   if (NewNumbering)
   {
@@ -668,7 +622,7 @@ wchar* VolNameToFirstName(const wchar *VolName,wchar *FirstName,bool NewNumberin
   else
   {
     // Old volume numbering scheme. Just set the extension to ".rar".
-    SetExt(FirstName,L"rar");
+    SetExt(FirstName,L"rar",MaxSize);
     VolNumStart=GetExt(FirstName);
   }
   if (!FileExist(FirstName))
@@ -677,8 +631,8 @@ wchar* VolNameToFirstName(const wchar *VolName,wchar *FirstName,bool NewNumberin
     // check if volume with same name and any other extension is available.
     // It can help in case of *.exe or *.sfx first volume.
     wchar Mask[NM];
-    wcscpy(Mask,FirstName);
-    SetExt(Mask,L"*");
+    wcsncpyz(Mask,FirstName,ASIZE(Mask));
+    SetExt(Mask,L"*",ASIZE(Mask));
     FindFile Find;
     Find.SetMask(Mask);
     FindData FD;
@@ -687,7 +641,7 @@ wchar* VolNameToFirstName(const wchar *VolName,wchar *FirstName,bool NewNumberin
       Archive Arc;
       if (Arc.Open(FD.Name,0) && Arc.IsArchive(true) && Arc.FirstVolume)
       {
-        wcscpy(FirstName,FD.Name);
+        wcsncpyz(FirstName,FD.Name,MaxSize);
         break;
       }
     }
@@ -698,7 +652,7 @@ wchar* VolNameToFirstName(const wchar *VolName,wchar *FirstName,bool NewNumberin
 
 
 #ifndef SFX_MODULE
-static void GenArcName(wchar *ArcName,wchar *GenerateMask,uint ArcNumber,bool &ArcNumPresent)
+static void GenArcName(wchar *ArcName,const wchar *GenerateMask,uint ArcNumber,bool &ArcNumPresent)
 {
   bool Prefix=false;
   if (*GenerateMask=='+')
@@ -821,7 +775,15 @@ static void GenArcName(wchar *ArcName,wchar *GenerateMask,uint ArcNumber,bool &A
     }
     const wchar *ChPtr=wcschr(MaskChars,toupperw(Mask[I]));
     if (ChPtr==NULL || QuoteMode)
+    {
       DateText[J]=Mask[I];
+#ifdef _WIN_ALL
+      // We do not allow ':' in Windows because of NTFS streams.
+      // Users had problems after specifying hh:mm mask.
+      if (DateText[J]==':')
+        DateText[J]='_';
+#endif
+    }
     else
     {
       size_t FieldPos=ChPtr-MaskChars;
@@ -856,7 +818,7 @@ static void GenArcName(wchar *ArcName,wchar *GenerateMask,uint ArcNumber,bool &A
 }
 
 
-void GenerateArchiveName(wchar *ArcName,size_t MaxSize,wchar *GenerateMask,bool Archiving)
+void GenerateArchiveName(wchar *ArcName,size_t MaxSize,const wchar *GenerateMask,bool Archiving)
 {
   // Must be enough space for archive name plus all stuff in mask plus
   // extra overhead produced by mask 'N' (archive number) characters.
@@ -925,11 +887,11 @@ bool GetWinLongPath(const wchar *Src,wchar *Dest,size_t MaxSize)
     return false;
   const wchar *Prefix=L"\\\\?\\";
   const size_t PrefixLength=4;
-  bool FullPath=IsDiskLetter(Src) && IsPathDiv(Src[2]);
+  bool FullPath=IsDriveLetter(Src) && IsPathDiv(Src[2]);
   size_t SrcLength=wcslen(Src);
   if (IsFullPath(Src)) // Paths in d:\path\name format.
   {
-    if (IsDiskLetter(Src))
+    if (IsDriveLetter(Src))
     {
       if (MaxSize<=PrefixLength+SrcLength)
         return false;
@@ -973,10 +935,27 @@ bool GetWinLongPath(const wchar *Src,wchar *Dest,size_t MaxSize)
         return false;
       wcsncpy(Dest,Prefix,PrefixLength);
       wcscpy(Dest+PrefixLength,CurDir);
+
+      if (Src[0]=='.' && IsPathDiv(Src[1])) // Remove leading .\ in pathname.
+        Src+=2;
+
       wcsncatz(Dest,Src,MaxSize);
       return true;
     }
   }
   return false;
 }
+
+
+// Convert Unix, OS X and Android decomposed chracters to Windows precomposed.
+void ConvertToPrecomposed(wchar *Name,size_t NameSize)
+{
+  wchar FileName[NM];
+  if (WinNT()>=WNT_VISTA && // MAP_PRECOMPOSED is not supported in XP.
+      FoldString(MAP_PRECOMPOSED,Name,-1,FileName,ASIZE(FileName))!=0)
+  {
+    FileName[ASIZE(FileName)-1]=0;
+    wcsncpyz(Name,FileName,NameSize);
+  }
+}
 #endif
