@@ -55,6 +55,7 @@ int main(int argc, char *argv[])
     }
     Cmd->AddArcName(ModuleName);
     Cmd->ParseDone();
+    Cmd->AbsoluteLinks=true; // If users runs SFX, he trusts an archive source.
 #else // !SFX_MODULE
     Cmd->ParseCommandLine(true,argc,argv);
     if (!Cmd->ConfigDisabled)
@@ -69,7 +70,8 @@ int main(int argc, char *argv[])
     ShutdownOnClose=Cmd->Shutdown;
 #endif
 
-    InitConsoleOptions(Cmd->MsgStream,Cmd->Sound);
+    uiInit(Cmd->Sound);
+    InitConsoleOptions(Cmd->MsgStream,Cmd->RedirectCharset);
     InitLogOptions(Cmd->LogName,Cmd->ErrlogCharset);
     ErrHandler.SetSilent(Cmd->AllYes || Cmd->MsgStream==MSG_NULL);
     ErrHandler.SetShutdown(Cmd->Shutdown);
@@ -82,7 +84,7 @@ int main(int argc, char *argv[])
   {
     ErrHandler.SetErrorCode(ErrCode);
   }
-  catch (std::bad_alloc)
+  catch (std::bad_alloc&)
   {
     ErrHandler.MemoryErrorMsg();
     ErrHandler.SetErrorCode(RARX_MEMORY);
