@@ -4,6 +4,9 @@
 #define  Min(x,y) (((x)<(y)) ? (x):(y))
 #define  Max(x,y) (((x)>(y)) ? (x):(y))
 
+// Universal replacement of abs function for non-int arguments.
+#define  Abs(x) (((x)<0) ? -(x):(x))
+
 #define  ASIZE(x) (sizeof(x)/sizeof(x[0]))
 
 // MAXPASSWORD is expected to be multiple of CRYPTPROTECTMEMORY_BLOCK_SIZE (16)
@@ -24,7 +27,7 @@
 // for those function calls where we do not need it.
 #define ignore_result(x) if (x)
 
-// Produce the value, which is larger than 'v' and aligned to 'a'.
+// Produce the value, which is equal or larger than 'v' and aligned to 'a'.
 #define ALIGN_VALUE(v,a) (size_t(v) + ( (~size_t(v) + 1) & (a - 1) ) )
 
 #endif
