@@ -4,7 +4,7 @@ int ToPercent(int64 N1,int64 N2)
 {
   if (N2<N1)
     return(100);
-  return(ToPercentUnlim(N1,N2));
+  return ToPercentUnlim(N1,N2);
 }
 
 
@@ -13,5 +13,7 @@ int ToPercentUnlim(int64 N1,int64 N2)
 {
   if (N2==0)
     return(0);
-  return((int)(N1*100/N2));
+  return (int)(N1*100/N2);
 }
+
+
