@@ -22,14 +22,17 @@ class RarTime
     // since 01.01.1601.
     uint64 itime;
   public:
-    RarTime();
+    RarTime() {Reset();}
 #ifdef _WIN_ALL
+    RarTime(FILETIME &ft) {*this=ft;}
     RarTime& operator =(FILETIME &ft);
     void GetWin32(FILETIME *ft);
 #endif
+    RarTime(time_t ut) {*this=ut;}
     RarTime& operator =(time_t ut);
     time_t GetUnix();
     bool operator == (RarTime &rt) {return itime==rt.itime;}
+    bool operator != (RarTime &rt) {return itime!=rt.itime;}
     bool operator < (RarTime &rt)  {return itime<rt.itime;}
     bool operator <= (RarTime &rt) {return itime<rt.itime || itime==rt.itime;}
     bool operator > (RarTime &rt)  {return itime>rt.itime;}
@@ -40,12 +43,12 @@ class RarTime
     void SetRaw(uint64 RawTime);
     uint GetDos();
     void SetDos(uint DosTime);
-    void GetText(wchar *DateStr,size_t MaxSize,bool FullYear,bool FullMS);
+    void GetText(wchar *DateStr,size_t MaxSize,bool FullMS);
     void SetIsoText(const wchar *TimeText);
     void SetAgeText(const wchar *TimeText);
     void SetCurrentTime();
     void Reset() {itime=0;}
-    bool IsSet() {return(itime!=0);}
+    bool IsSet() {return itime!=0;}
 };
 
 const wchar *GetMonthName(int Month);
