@@ -26,13 +26,17 @@ bool WideToChar(const wchar *Src,char *Dest,size_t DestSize)
   if (WideCharToMultiByte(CP_ACP,0,Src,-1,Dest,(int)DestSize,NULL,NULL)==0)
     RetCode=false;
 
-#elif defined(_APPLE)
+// wcstombs is broken in Android NDK r9.
+#elif defined(_APPLE) || defined(_ANDROID)
   WideToUtf(Src,Dest,DestSize);
 
 #elif defined(MBFUNCTIONS)
   if (!WideToCharMap(Src,Dest,DestSize,RetCode))
   {
-    size_t ResultingSize=wcstombs(Dest,Src,DestSize);
+    mbstate_t ps; // Use thread safe external state based functions.
+    memset (&ps, 0, sizeof(ps));
+    const wchar *SrcParam=Src; // wcsrtombs can change the pointer.
+    size_t ResultingSize=wcsrtombs(Dest,&SrcParam,DestSize,&ps);
     if (ResultingSize==(size_t)-1)
       RetCode=false;
     if (ResultingSize==0 && *Src!=0)
@@ -69,11 +73,15 @@ bool CharToWide(const char *Src,wchar *Dest,size_t DestSize)
   if (MultiByteToWideChar(CP_ACP,0,Src,-1,Dest,(int)DestSize)==0)
     RetCode=false;
 
-#elif defined(_APPLE)
+// mbstowcs is broken in Android NDK r9.
+#elif defined(_APPLE) || defined(_ANDROID)
   UtfToWide(Src,Dest,DestSize);
 
 #elif defined(MBFUNCTIONS)
-  size_t ResultingSize=mbstowcs(Dest,Src,DestSize);
+  mbstate_t ps;
+  memset (&ps, 0, sizeof(ps));
+  const char *SrcParam=Src; // mbsrtowcs can change the pointer.
+  size_t ResultingSize=mbsrtowcs(Dest,&SrcParam,DestSize,&ps);
   if (ResultingSize==(size_t)-1)
     RetCode=false;
   if (ResultingSize==0 && *Src!=0)
@@ -81,7 +89,6 @@ bool CharToWide(const char *Src,wchar *Dest,size_t DestSize)
 
   if (RetCode==false && DestSize>1)
     CharToWideMap(Src,Dest,DestSize,RetCode);
-
 #else
   for (int I=0;I<DestSize;I++)
   {
@@ -103,7 +110,7 @@ bool CharToWide(const char *Src,wchar *Dest,size_t DestSize)
 }
 
 
-#if defined(_UNIX) && defined(MBFUNCTIONS)
+#if defined(_UNIX) && defined(MBFUNCTIONS) && !defined(_ANDROID)
 // Convert and restore mapped inconvertible Unicode characters. 
 // We use it for extended ASCII names in Unix.
 bool WideToCharMap(const wchar *Src,char *Dest,size_t DestSize,bool &Success)
@@ -133,12 +140,13 @@ bool WideToCharMap(const wchar *Src,char *Dest,size_t DestSize,bool &Success)
       Dest[DestPos++]=char(uint(Src[SrcPos++])-MapAreaStart);
     else
     {
-      ignore_result( wctomb(NULL,0) ); // Reset shift state.
-      if (wctomb(Dest+DestPos,Src[SrcPos])==-1)
+      mbstate_t ps;
+      memset(&ps,0,sizeof(ps));
+      if (wcrtomb(Dest+DestPos,Src[SrcPos],&ps)==-1)
         Success=false;
       SrcPos++;
-      ignore_result( mblen(NULL,0) ); // Reset shift state.
-      int Length=mblen(Dest+DestPos,MB_CUR_MAX);
+      memset(&ps,0,sizeof(ps));
+      int Length=mbrlen(Dest+DestPos,MB_CUR_MAX,&ps);
       DestPos+=Max(Length,1);
     }
   }
@@ -147,7 +155,7 @@ bool WideToCharMap(const wchar *Src,char *Dest,size_t DestSize,bool &Success)
 #endif
 
 
-#if defined(_UNIX) && defined(MBFUNCTIONS)
+#if defined(_UNIX) && defined(MBFUNCTIONS) && !defined(_ANDROID)
 // Convert and map inconvertible Unicode characters. 
 // We use it for extended ASCII names in Unix.
 void CharToWideMap(const char *Src,wchar *Dest,size_t DestSize,bool &Success)
@@ -166,8 +174,9 @@ void CharToWideMap(const char *Src,wchar *Dest,size_t DestSize,bool &Success)
       Success=true;
       break;
     }
-    ignore_result( mbtowc(NULL,NULL,0) ); // Reset shift state.
-    if (mbtowc(Dest+DestPos,Src+SrcPos,MB_CUR_MAX)==-1)
+    mbstate_t ps;
+    memset(&ps,0,sizeof(ps));
+    if (mbrtowc(Dest+DestPos,Src+SrcPos,MB_CUR_MAX,&ps)==-1)
     {
       // For security reasons we do not want to map low ASCII characters,
       // so we do not have additional .. and path separator codes.
@@ -187,8 +196,8 @@ void CharToWideMap(const char *Src,wchar *Dest,size_t DestSize,bool &Success)
     }
     else
     {
-      ignore_result( mblen(NULL,0) ); // Reset shift state.
-      int Length=mblen(Src+SrcPos,MB_CUR_MAX);
+      memset(&ps,0,sizeof(ps));
+      int Length=mbrlen(Src+SrcPos,MB_CUR_MAX,&ps);
       SrcPos+=Max(Length,1);
       DestPos++;
     }
@@ -373,14 +382,18 @@ int wcsicomp(const wchar *s1,const wchar *s2)
 #ifdef _WIN_ALL
   return CompareStringW(LOCALE_USER_DEFAULT,NORM_IGNORECASE|SORT_STRINGSORT,s1,-1,s2,-1)-2;
 #else
-  while (towupper(*s1)==towupper(*s2))
+  while (true)
   {
+    wchar u1 = towupper(*s1);
+    wchar u2 = towupper(*s2);
+    if (u1 != u2)
+      return u1 < u2 ? -1 : 1;
     if (*s1==0)
-      return 0;
+      break;
     s1++;
     s2++;
   }
-  return s1 < s2 ? -1 : 1;
+  return 0;
 #endif
 }
 
@@ -393,27 +406,45 @@ int wcsnicomp(const wchar *s1,const wchar *s2,size_t n)
   // to real string length.
   size_t l1=Min(wcslen(s1)+1,n);
   size_t l2=Min(wcslen(s2)+1,n);
-  return(CompareStringW(LOCALE_USER_DEFAULT,NORM_IGNORECASE|SORT_STRINGSORT,s1,(int)l1,s2,(int)l2)-2);
+  return CompareStringW(LOCALE_USER_DEFAULT,NORM_IGNORECASE|SORT_STRINGSORT,s1,(int)l1,s2,(int)l2)-2;
 #else
   if (n==0)
     return 0;
-  while (towupper(*s1)==towupper(*s2))
+  while (true)
   {
+    wchar u1 = towupper(*s1);
+    wchar u2 = towupper(*s2);
+    if (u1 != u2)
+      return u1 < u2 ? -1 : 1;
     if (*s1==0 || --n==0)
-      return 0;
+      break;
     s1++;
     s2++;
   }
-  return s1 < s2 ? -1 : 1;
+  return 0;
 #endif
 }
 
 
+const wchar_t* wcscasestr(const wchar_t *str, const wchar_t *search)
+{
+  for (size_t i=0;str[i]!=0;i++)
+    for (size_t j=0;;j++)
+    {
+      if (search[j]==0)
+        return str+i;
+      if (tolowerw(str[i+j])!=tolowerw(search[j]))
+        break;
+    }
+  return NULL;
+}
+
+
 #ifndef SFX_MODULE
 wchar* wcslower(wchar *s)
 {
 #ifdef _WIN_ALL
-  CharLowerW(s);
+  CharLower(s);
 #else
   for (wchar *c=s;*c!=0;c++)
     *c=towlower(*c);
@@ -427,7 +458,7 @@ wchar* wcslower(wchar *s)
 wchar* wcsupper(wchar *s)
 {
 #ifdef _WIN_ALL
-  CharUpperW(s);
+  CharUpper(s);
 #else
   for (wchar *c=s;*c!=0;c++)
     *c=towupper(*c);
@@ -437,10 +468,15 @@ wchar* wcsupper(wchar *s)
 #endif
 
 
+
+
 int toupperw(int ch)
 {
 #ifdef _WIN_ALL
-  return (int)CharUpperW((wchar *)ch);
+  // CharUpper is more reliable than towupper in Windows, which seems to be
+  // C locale dependent even in Unicode version. For example, towupper failed
+  // to convert lowercase Russian characters.
+  return (int)CharUpper((wchar *)ch);
 #else
   return towupper(ch);
 #endif
@@ -450,28 +486,36 @@ int toupperw(int ch)
 int tolowerw(int ch)
 {
 #ifdef _WIN_ALL
-  return (int)CharLowerW((wchar *)ch);
+  // CharLower is more reliable than towlower in Windows.
+  // See comment for towupper above.
+  return (int)CharLower((wchar *)ch);
 #else
   return towlower(ch);
 #endif
 }
 
 
-uint atoiw(const wchar *s)
+int atoiw(const wchar *s)
 {
-  return (uint)atoilw(s);
+  return (int)atoilw(s);
 }
 
 
-uint64 atoilw(const wchar *s)
+int64 atoilw(const wchar *s)
 {
-  uint64 n=0;
+  int sign=1;
+  if (*s=='-')
+  {
+    s++;
+    sign=-1;
+  }
+  int64 n=0;
   while (*s>='0' && *s<='9')
   {
     n=n*10+(*s-'0');
     s++;
   }
-  return n;
+  return sign*n;
 }
 
 
