@@ -1,28 +1,11 @@
 #include "rar.hpp"
 
-void Unpack::CopyString20(unsigned int Length,unsigned int Distance)
+void Unpack::CopyString20(uint Length,uint Distance)
 {
   LastDist=OldDist[OldDistPtr++ & 3]=Distance;
   LastLength=Length;
   DestUnpSize-=Length;
-
-  unsigned int DestPtr=UnpPtr-Distance;
-  if (DestPtr<MAXWINSIZE-300 && UnpPtr<MAXWINSIZE-300)
-  {
-    Window[UnpPtr++]=Window[DestPtr++];
-    Window[UnpPtr++]=Window[DestPtr++];
-    while (Length>2)
-    {
-      Length--;
-      Window[UnpPtr++]=Window[DestPtr++];
-    }
-  }
-  else
-    while (Length--)
-    {
-      Window[UnpPtr]=Window[DestPtr++ & MAXWINMASK];
-      UnpPtr=(UnpPtr+1) & MAXWINMASK;
-    }
+  CopyString(Length,Distance);
 }
 
 
@@ -51,20 +34,20 @@ void Unpack::Unpack20(bool Solid)
 
   while (DestUnpSize>=0)
   {
-    UnpPtr&=MAXWINMASK;
+    UnpPtr&=MaxWinMask;
 
-    if (InAddr>ReadTop-30)
+    if (Inp.InAddr>ReadTop-30)
       if (!UnpReadBuf())
         break;
-    if (((WrPtr-UnpPtr) & MAXWINMASK)<270 && WrPtr!=UnpPtr)
+    if (((WrPtr-UnpPtr) & MaxWinMask)<270 && WrPtr!=UnpPtr)
     {
-      OldUnpWriteBuf();
+      UnpWriteBuf20();
       if (Suspended)
         return;
     }
     if (UnpAudioBlock)
     {
-      int AudioNumber=DecodeNumber((struct Decode *)&MD[UnpCurChannel]);
+      int AudioNumber=DecodeNumber(Inp,&MD[UnpCurChannel]);
 
       if (AudioNumber==256)
       {
@@ -79,7 +62,7 @@ void Unpack::Unpack20(bool Solid)
       continue;
     }
 
-    int Number=DecodeNumber((struct Decode *)&LD);
+    int Number=DecodeNumber(Inp,&BlockTables.LD);
     if (Number<256)
     {
       Window[UnpPtr++]=(byte)Number;
@@ -91,16 +74,16 @@ void Unpack::Unpack20(bool Solid)
       int Length=LDecode[Number-=270]+3;
       if ((Bits=LBits[Number])>0)
       {
-        Length+=getbits()>>(16-Bits);
-        addbits(Bits);
+        Length+=Inp.getbits()>>(16-Bits);
+        Inp.addbits(Bits);
       }
 
-      int DistNumber=DecodeNumber((struct Decode *)&DD);
+      int DistNumber=DecodeNumber(Inp,&BlockTables.DD);
       unsigned int Distance=DDecode[DistNumber]+1;
       if ((Bits=DBits[DistNumber])>0)
       {
-        Distance+=getbits()>>(16-Bits);
-        addbits(Bits);
+        Distance+=Inp.getbits()>>(16-Bits);
+        Inp.addbits(Bits);
       }
 
       if (Distance>=0x2000)
@@ -127,12 +110,12 @@ void Unpack::Unpack20(bool Solid)
     if (Number<261)
     {
       unsigned int Distance=OldDist[(OldDistPtr-(Number-256)) & 3];
-      int LengthNumber=DecodeNumber((struct Decode *)&RD);
+      int LengthNumber=DecodeNumber(Inp,&BlockTables.RD);
       int Length=LDecode[LengthNumber]+2;
       if ((Bits=LBits[LengthNumber])>0)
       {
-        Length+=getbits()>>(16-Bits);
-        addbits(Bits);
+        Length+=Inp.getbits()>>(16-Bits);
+        Inp.addbits(Bits);
       }
       if (Distance>=0x101)
       {
@@ -152,39 +135,55 @@ void Unpack::Unpack20(bool Solid)
       unsigned int Distance=SDDecode[Number-=261]+1;
       if ((Bits=SDBits[Number])>0)
       {
-        Distance+=getbits()>>(16-Bits);
-        addbits(Bits);
+        Distance+=Inp.getbits()>>(16-Bits);
+        Inp.addbits(Bits);
       }
       CopyString20(2,Distance);
       continue;
    }
   }
   ReadLastTables();
-  OldUnpWriteBuf();
+  UnpWriteBuf20();
+}
+
+
+void Unpack::UnpWriteBuf20()
+{
+  if (UnpPtr!=WrPtr)
+    UnpSomeRead=true;
+  if (UnpPtr<WrPtr)
+  {
+    UnpIO->UnpWrite(&Window[WrPtr],-(int)WrPtr & MaxWinMask);
+    UnpIO->UnpWrite(Window,UnpPtr);
+    UnpAllBuf=true;
+  }
+  else
+    UnpIO->UnpWrite(&Window[WrPtr],UnpPtr-WrPtr);
+  WrPtr=UnpPtr;
 }
 
 
 bool Unpack::ReadTables20()
 {
   byte BitLength[BC20];
-  unsigned char Table[MC20*4];
+  byte Table[MC20*4];
   int TableSize,N,I;
-  if (InAddr>ReadTop-25)
+  if (Inp.InAddr>ReadTop-25)
     if (!UnpReadBuf())
       return(false);
-  unsigned int BitField=getbits();
+  uint BitField=Inp.getbits();
   UnpAudioBlock=(BitField & 0x8000);
 
   if (!(BitField & 0x4000))
     memset(UnpOldTable20,0,sizeof(UnpOldTable20));
-  addbits(2);
+  Inp.addbits(2);
 
   if (UnpAudioBlock)
   {
     UnpChannels=((BitField>>12) & 3)+1;
     if (UnpCurChannel>=UnpChannels)
       UnpCurChannel=0;
-    addbits(2);
+    Inp.addbits(2);
     TableSize=MC20*UnpChannels;
   }
   else
@@ -192,17 +191,17 @@ bool Unpack::ReadTables20()
 
   for (I=0;I<BC20;I++)
   {
-    BitLength[I]=(byte)(getbits() >> 12);
-    addbits(4);
+    BitLength[I]=(byte)(Inp.getbits() >> 12);
+    Inp.addbits(4);
   }
-  MakeDecodeTables(BitLength,(struct Decode *)&BD,BC20);
+  MakeDecodeTables(BitLength,&BlockTables.BD,BC20);
   I=0;
   while (I<TableSize)
   {
-    if (InAddr>ReadTop-5)
+    if (Inp.InAddr>ReadTop-5)
       if (!UnpReadBuf())
-        return(false);
-    int Number=DecodeNumber((struct Decode *)&BD);
+        return false;
+    int Number=DecodeNumber(Inp,&BlockTables.BD);
     if (Number<16)
     {
       Table[I]=(Number+UnpOldTable20[I]) & 0xf;
@@ -211,40 +210,41 @@ bool Unpack::ReadTables20()
     else
       if (Number==16)
       {
-        N=(getbits() >> 14)+3;
-        addbits(2);
-        while (N-- > 0 && I<TableSize)
-        {
-          Table[I]=Table[I-1];
-          I++;
-        }
+        N=(Inp.getbits() >> 14)+3;
+        Inp.addbits(2);
+        if (I>0)
+          while (N-- > 0 && I<TableSize)
+          {
+            Table[I]=Table[I-1];
+            I++;
+          }
       }
       else
       {
         if (Number==17)
         {
-          N=(getbits() >> 13)+3;
-          addbits(3);
+          N=(Inp.getbits() >> 13)+3;
+          Inp.addbits(3);
         }
         else
         {
-          N=(getbits() >> 9)+11;
-          addbits(7);
+          N=(Inp.getbits() >> 9)+11;
+          Inp.addbits(7);
         }
         while (N-- > 0 && I<TableSize)
           Table[I++]=0;
       }
   }
-  if (InAddr>ReadTop)
+  if (Inp.InAddr>ReadTop)
     return(true);
   if (UnpAudioBlock)
     for (I=0;I<UnpChannels;I++)
-      MakeDecodeTables(&Table[I*MC20],(struct Decode *)&MD[I],MC20);
+      MakeDecodeTables(&Table[I*MC20],&MD[I],MC20);
   else
   {
-    MakeDecodeTables(&Table[0],(struct Decode *)&LD,NC20);
-    MakeDecodeTables(&Table[NC20],(struct Decode *)&DD,DC20);
-    MakeDecodeTables(&Table[NC20+DC20],(struct Decode *)&RD,RC20);
+    MakeDecodeTables(&Table[0],&BlockTables.LD,NC20);
+    MakeDecodeTables(&Table[NC20],&BlockTables.DD,DC20);
+    MakeDecodeTables(&Table[NC20+DC20],&BlockTables.RD,RC20);
   }
   memcpy(UnpOldTable20,Table,sizeof(UnpOldTable20));
   return(true);
@@ -253,14 +253,14 @@ bool Unpack::ReadTables20()
 
 void Unpack::ReadLastTables()
 {
-  if (ReadTop>=InAddr+5)
+  if (ReadTop>=Inp.InAddr+5)
     if (UnpAudioBlock)
     {
-      if (DecodeNumber((struct Decode *)&MD[UnpCurChannel])==256)
+      if (DecodeNumber(Inp,&MD[UnpCurChannel])==256)
         ReadTables20();
     }
     else
-      if (DecodeNumber((struct Decode *)&LD)==269)
+      if (DecodeNumber(Inp,&BlockTables.LD)==269)
         ReadTables20();
 }
 
