@@ -78,7 +78,7 @@ void ExtractStreams20(Archive &Arc,const wchar *FileName)
 
 
 #ifdef _WIN_ALL
-void ExtractStreams(Archive &Arc,const wchar *FileName)
+void ExtractStreams(Archive &Arc,const wchar *FileName,bool TestMode)
 {
   wchar FullName[NM+2];
   if (FileName[0]!=0 && FileName[1]==0)
@@ -101,6 +101,12 @@ void ExtractStreams(Archive &Arc,const wchar *FileName)
     return;
   }
 
+  if (TestMode)
+  {
+    Arc.ReadSubData(NULL,NULL);
+    return;
+  }
+
   wcsncatz(FullName,StreamName,ASIZE(FullName));
 
   FindData fd;
